@echo off
setlocal
rem #################################################################
rem
rem  Steuerskript zum Anlegen und Verwalten der Schattenkopien
rem
rem #################################################################

SET glob_konfig_bat=%~dp0..\konfig.bat

if exist %glob_konfig_bat% (
	call %glob_konfig_bat%
	cd %~dp0
) else (
	echo %glob_konfig_bat% nicht gefunden.
	goto :eof	
)

rem #################################
rem 
rem  die folgenden Variablen werden 
rem  aus KONFIG.BAT bentigt
rem
rem  %glob_completelog%
rem  %glob_logdir%
rem  %glob_backup_volume%
rem  %glob_shadow_volume% 
rem  %glob_shadow_log%
rem  %glob_shadow_report%
rem  %glob_exposedir%
rem  %glob_free_percentage%
rem  %glob_warn_percentage%
rem  %glob_cmd_shadowcopy_msg%
rem
rem #################################


rem #########################################################################
rem # ab hier beginnt das Skript
rem #########################################################################
rem
rem  Datum im ISO-Format 

set BACKUP_DATE=%date:~6,4%-%date:~3,2%-%date:~0,2%

rem
rem  Uhrzeit formatieren (ersetze ":" durch "-")
rem  vorangehende Null evtl. ergnzen

set BACKUP_TIME=%time:~0,8%
set BACKUP_TIME=%BACKUP_TIME::=-%
set BACKUP_TIME=%BACKUP_TIME: =0%

rem ############################################################################
rem
rem  Wochentag herausfinden (englische Abkrzungen mappen)

for /f "tokens=1" %%i in ('%now_exe%') do set BACKUP_WEEKDAY=%%i

if %BACKUP_WEEKDAY%==Mon set BACKUP_WEEKDAY=Mo
if %BACKUP_WEEKDAY%==Tue set BACKUP_WEEKDAY=Di
if %BACKUP_WEEKDAY%==Wed set BACKUP_WEEKDAY=Mi
if %BACKUP_WEEKDAY%==Thu set BACKUP_WEEKDAY=Do
if %BACKUP_WEEKDAY%==Fri set BACKUP_WEEKDAY=Fr
if %BACKUP_WEEKDAY%==Sat set BACKUP_WEEKDAY=Sa
if %BACKUP_WEEKDAY%==Sun set BACKUP_WEEKDAY=So


rem ############################################################################
rem
rem eindeutigen Verzeichnisnamen festlegen, dabei Leerzeichen ersetzen


set UNIQUE_DIR=%BACKUP_DATE%_%BACKUP_WEEKDAY%_%BACKUP_TIME%
set UNIQUE_DIR=%UNIQUE_DIR: =_%

set EXPOSEDIR=%glob_exposedir%%UNIQUE_DIR%

if exist %EXPOSEDIR% (
	echo %EXPOSEDIR% existiert bereits; Vorgang abbrechen
	goto :eof
)

rem ############################################################################
rem
rem  Speicherplatz auf SHADOW_VOLUME prfen.
rem
rem  Hinweis:
rem  Falls unter FREE_PERCENTAGE freier Speicherplatz zur verfgung steht, wird
rem  die lteste Schattenkopie des BACKUP_VOLUMES gelscht
rem
rem  Falls dem Schattenkopiedienst (VSS) whrend des Erzeugens einer Schattenkopie
rem  der Festplattenplatz ausgeht gehen unkontrolliert die ltesten Schattenkopien
rem  defekt und die aktuelle kann u.U. auch nicht richtig geschrieben werden.
rem  Deshalb wird WARN_PERCENTAGE geprft und ggf. eine Email geschickt.
rem  
rem  *************************************************
rem  | VORSICHT! Das SHADOW_VOLUME sollte nicht als  |
rem  | Zwischenspeicher fr Daten missbraucht werden |
rem  *************************************************
rem
rem  Volumedaten abfragen

set TEMPFILE=zz_tmp_diskfree.txt

fsutil volume diskfree %glob_shadow_volume% | findstr /n ":" > %TEMPFILE%

for /f "delims=: tokens=1,3 skip=1" %%a in (%TEMPFILE%) do (
	if %%a == 2 set VOLSPACE=%%b
	if %%a == 3 set VOLFREE_BEF=%%b
)
del %TEMPFILE%

rem ############################################################################
rem
rem  Speicherplatz berechnen.
rem
rem  Die Textstrings aus der Rckgabe des fsutil-Befehls mussen noch formatiert werden:
rem
rem  Das fhrende Leerzeichen und die letzten sechs Stellen der in Bytes angegebenen
rem  Daten werden abgeschnitten. Dadurch liegen die Grenangaben in Megabytes vor und
rem  die Daten knnen (rudimentr) berechnet werden.
rem
rem  Hintergrund: CMD kann nur 32bit-Ganzzahlen verarbeiten, die zu erwartenden Rundungs-
rem  fehler sind aber bei FREE_PERCENTAGE >= 5 absolut venachlssigbar.
rem
rem  Achtung: Die Angaben erfolgen in "echten" Megabytes (Faktor 10^6 _nicht_ 2^20 ) und sind
rem  nicht auf die letzte Stelle gerundet, die Dezimalstellen werden einfach "abgeschnitten".
rem

set VOLSPACE=%VOLSPACE:~1,-6%
set VOLFREE_BEF=%VOLFREE_BEF:~1,-6%


rem  ################################
rem  Berechnung der Festplattenlimits
rem  ################################


SET /A VOLIMIT=%VOLSPACE% / 1000 * %glob_free_percentage%
SET /A WARLIMIT=%VOLSPACE% / 1000 * %glob_warn_percentage%

rem
rem ### DEBUG
rem set /a VOLFREE_BEF=%VOLIMIT%-1

rem ###

echo freier Speicherplatz auf %glob_shadow_volume% %VOLFREE_BEF% MByte; Limit %VOLIMIT% MByte

echo. 							>> %glob_shadow_report%
echo -----						>> %glob_shadow_report%
echo %DATE% %TIME:~0,8% neue Schattenkopie anlegen	>> %glob_shadow_report%
echo.							>> %glob_shadow_report%

echo Speicherplatz auf %glob_shadow_volume% prfen:	>> %glob_shadow_report%
echo  - frei:  %VOLFREE_BEF% MBytes			>> %glob_shadow_report%
echo  - Limit: %VOLIMIT% MBytes   			>> %glob_shadow_report%
echo  - Warn:  %WARLIMIT% MBytes			>> %glob_shadow_report%


echo ---------------------	> %glob_cmd_shadowcopy_msg%
echo ##  Schattenkopie  ##	>> %glob_cmd_shadowcopy_msg%



rem  #######################################################
rem  Prfen, ob Warnmeldung per Email verschickt werden soll
rem  #######################################################

if %VOLFREE_BEF% leq %WARLIMIT% (
	echo ---------------------------------------------------
	echo Festplattenkapazitaet fuer Schattenkopien kritisch!
	echo ---------------------------------------------------
	goto :warnmsg
)

:procede_normal
rem  ##############################
rem  gengend freier Speicherplatz?
rem  ##############################

if %VOLFREE_BEF% geq %VOLIMIT% goto :no_delete_shadow

rem ############################################################################
:delete_shadow
rem ##################
rem #  Lschroutine  #
rem ##################
rem


echo Freier Speicherplatz auf %glob_shadow_volume% unter %VOLIMIT% MByte !!!

echo Nicht gengend freier Speicher auf %glob_shadow_volume% >> %glob_shadow_report%
 
echo Lsche alteste Schattenkopie >> %glob_shadow_report%


rem ############################################################################
rem
rem  lteste Schattenkopie auf BACKUP_VOLUME lschen und Ausgabe loggen fr Shadow-ID

set TEMPFILE=zz_tmp_shadowid.txt

echo Deleting oldest shadow copy for %glob_backup_volume%

%vshadow_exe% -do=%glob_backup_volume% | findstr "Deleting" > %TEMPFILE%


rem ############################################################################
rem
rem  Snapshot-ID der gelschten Schattenkopie auslesen


for /f "tokens=5" %%a in (%TEMPFILE%) do (
	set SHID=%%a
)

del %TEMPFILE%

echo Schattenkopie-Snapshot %SHID% gelscht		>> %glob_shadow_report%
echo. 							>> %glob_shadow_report%
echo lsche zugehriges Bereitstellungsverzeichnis	>> %glob_shadow_report%

echo.
echo Searching for %SHID% in %glob_shadow_expose_log%
echo.

set TEMPFILE=zz_tmp_lastdelshadow.txt

findstr "%SHID%" %glob_shadow_expose_log% > %TEMPFILE%

if /i %ERRORLEVEL% gtr 0 (
	echo. 								>> %glob_shadow_report%
	echo Bereitstellungsverzeichnis fr Schattenkopie nicht gefunden! >> %glob_shadow_report%
	echo  - Datei %glob_shadow_expose_log% berprfen
	echo  - unter %glob_exposedir% nach nicht verknpften Schattenkopieverzeichissen suchen >> %glob_shadow_report%
	echo. 			>> %glob_shadow_report%
	goto :procede_with_dskchk
)

for /f %%a in (%TEMPFILE%) do (
	echo expose directory %%a is to be deleted
	rmdir /s /q %%a
	if %ERRORLEVEL% == 0 (
		echo %%a gelscht >> %glob_shadow_report%
		echo Snapshot %SHID% unter %%a gelscht >> %glob_cmd_shadowcopy_msg%
	)
	if %ERRORLEVEL% == 1 (
		echo %%a konnte nicht gelscht werden		>> %glob_shadow_report%
		echo Snapshot %SHID% gelscht			 >> %glob_cmd_shadowcopy_msg%
		echo   - FEHLER: %%a konnte nicht gelscht werden >> %glob_cmd_shadowcopy_msg%
	)
)
del %TEMPFILE%

rem ############################################################################
rem
rem  Verzeichnis-Eintrag aus %glob_shadow_expose_log% lschen
rem  

set TEMPFILE=zz_temp_shadowexposelog.txt

findstr /V "%SHID%" %glob_shadow_expose_log% > %TEMPFILE%
type %TEMPFILE% > %glob_shadow_expose_log%

del %TEMPFILE%

rem ############################################################################
rem
:procede_with_dskchk
rem
rem  Abfragen, ob Speicherplatz ber Limit.
rem  Falls nicht, dann letzen Schattenkopie-Snapshot lschen!
rem
rem  Volumedaten abfragen

set TEMPFILE=zz_tmp_diskfree.txt
set VOLFREE_AD=%VOLFREE_BEF%

fsutil volume diskfree %glob_shadow_volume% | findstr /n ":" > %TEMPFILE%

for /f "delims=: tokens=1,3 skip=2" %%a in (%TEMPFILE%) do (
	if %%a == 3 set VOLFREE_BEF=%%b
)
del %TEMPFILE%


set VOLFREE_BEF=%VOLFREE_BEF:~1,-6%

echo. 							>> %glob_shadow_report%
echo Speicherplatz auf %glob_shadow_volume% prfen: 	>> %glob_shadow_report%
echo  - frei:  %VOLFREE_BEF% MByte			>> %glob_shadow_report%
echo  - Limit: %VOLIMIT% MByte				>> %glob_shadow_report%

set /A SPACEGAIN=%VOLFREE_BEF% - %VOLFREE_AD%

echo - durch gelschte Schattenkopie freigegebener Speicherplatz: %SPACEGAIN% MB >> %glob_shadow_report%

rem  #######################################
rem  Loggen der Snapshotgre im COMPLETELOG

echo %DATE% %TIME:~0,8% Schattenkopie gelscht: %SPACEGAIN% MBytes >> %glob_completelog%

rem  #######################################


if %VOLFREE_BEF% leq %VOLIMIT% goto :delete_shadow

:no_delete_shadow

echo Gengend freier Speicherplatz auf %glob_shadow_volume%	>> %glob_shadow_report%
echo. 								>> %glob_shadow_report%
echo Erzeuge Schattenkopie von %glob_backup_volume% 		>> %glob_shadow_report%


echo. >> %glob_cmd_shadowcopy_msg%
echo freier Speicherplatz auf %glob_shadow_volume% %VOLFREE_BEF% MB  >> %glob_cmd_shadowcopy_msg%

rem ############################################################################
rem ############################################################################
rem
rem  Verzeichnis anlegen, worin die Schattenkopie verfgbar gemacht werden kann

md %EXPOSEDIR%


rem
rem Aufruf des Skripts zum Anfertigen und Bereitstellen der Schattenkopie

echo  - %TIME:~0,8% lege Schattenkopie an >> %glob_shadow_report%

call CreateShadow %glob_backup_volume% %EXPOSEDIR%

rem ############################################################################
rem ############################################################################

rem
rem  Prfe, ob Schattenkopie erfolgreich angelegt und bereitgestellt wurde
rem  Falls die Prfung einen Fehler ergibt, heisst das nicht automatisch,
rem  dass die Schattenkopie nicht angefertigt wurde, lediglich die Bereit-
rem  stellung unter %glob_exposedir% schlug fehl.

%vshadow_exe% -q | findstr %EXPOSEDIR% > nul

if /i %ERRORLEVEL% geq 1 (
	echo FEHLER beim Anlegen der Schattenkopie >> %glob_shadow_report%
	echo Schattenkopie nicht unter %EXPOSEDIR% verfgbar >> %glob_shadow_report%
	echo FEHLER beim Anlegen der Schattenkopie >> %glob_cmd_shadowcopy_msg%	
) else (
	echo  - %TIME:~0,8% Schattenkopie erfolgreich angelegt >> %glob_shadow_report%
	echo Schattenkopie ist unter %EXPOSEDIR% verfgbar >> %glob_shadow_report%
	echo Schattenkopie ist unter %EXPOSEDIR% verfgbar >> %glob_cmd_shadowcopy_msg%
)


rem ############################################################################
rem
rem  Der durchschnittliche Speicherplatzverbrauch aller angefertigten
rem  Schattenkopien kann jetzt berechnet werden.
rem
rem  Anm: Der verbrauchte Speicherplatz einer Schattenkopie kann nicht genau
rem       berechnet werden, da ein asynchroner Schreibprozess im Hintergrund
rem       u.U. ber mehrere Stunden ttig ist bis die Schattenkopie fertig 
rem       geschrieben wurde.
rem
rem
rem  Volumedaten abfragen

set TEMPFILE=zz_tmp_diskfree.txt

fsutil volume diskfree %glob_shadow_volume% | findstr /n ":" > %TEMPFILE%

for /f "delims=: tokens=1,3 skip=1" %%a in (%TEMPFILE%) do (
	if %%a == 3 set VOLFREE_AFT=%%b
)
del %TEMPFILE%

set VOLFREE_AFT=%VOLFREE_AFT:~1,-6%

echo.
echo freier Speicherplatz vor der Schattenkopie: %VOLFREE_BEF% MByte
echo.

echo. >> %glob_shadow_report%
echo %DATE% %TIME:~0,8% Vorgang abgeschlossen >> %glob_shadow_report%
echo ----- >> %glob_shadow_report%
echo. >> %glob_shadow_report%



rem ############################################################################
rem
rem  Report in Logdatei schreiben

type %glob_shadow_report% >> %glob_shadow_log%

goto :eof

:warnmsg
	
	if %glob_free_percentage% lss 10 (
		set gfpWert=0,%glob_free_percentage%
	) else (
		if %glob_free_percentage% lss 100 (
			set gfpWert=%glob_free_percentage:~0,1%,%glob_free_percentage:~1,1%
		) else (
			if %glob_free_percentage% lss 1000 (
				set gfpWert=%glob_free_percentage:~0,2%,%glob_free_percentage:~2,1%
			)
		)
	)

	SET _tmp_msg_file=%glob_logdir%zz_tmp_warnmail.txt
	echo ############################################			> %_tmp_msg_file%
	echo # 	Backupserver %COMPUTERNAME%					>> %_tmp_msg_file%
	echo #									>> %_tmp_msg_file%
	echo #  Warnung: Festplattenplatz Schattenkopien			>> %_tmp_msg_file%
	echo #									>> %_tmp_msg_file%
	echo ############################################			>> %_tmp_msg_file%
	echo.									>> %_tmp_msg_file%
	echo Der freie Speicherplatz fr Schattenkopien ist unter die Warn-	>> %_tmp_msg_file%
	echo schwelle von %WARLIMIT% MBytes gesunken.				>> %_tmp_msg_file%
	echo Um einen potentiellen, unkalkulierbaren Datenverlust zu vermeiden,	>> %_tmp_msg_file%
	echo sollte der Wert fr das Limit erhht werden.			>> %_tmp_msg_file%
	echo.									>> %_tmp_msg_file%
	echo Derzeitiges Limit: %VOLIMIT% 					>> %_tmp_msg_file%
	echo dies entspricht ca. %gfpWert% Prozent von %glob_shadow_volume%	>> %_tmp_msg_file%
	echo.									>> %_tmp_msg_file%
	echo Der Wert von GLOB_FREE_PERCENTAGE sollte in der KONFIG.BAT um 1	>> %_tmp_msg_file%
	echo erhht werden.	 						>> %_tmp_msg_file% 
	echo.									>> %_tmp_msg_file%
	call %glob_cmd_sendemail% "Schattenkopie-Warnung" %_tmp_msg_file%
	del %_tmp_msg_file%
	goto :procede_normal

:eof
