@ECHO off
setlocal
setlocal EnableDelayedExpansion

rem  ###################
rem  
rem  robocopy_task.bat
rem  
rem  Dies ist ein Hilfsscript fuer _shares_spiegeln.bat
rem  Es startet die einzelnen Robocopyinstanzen.
rem  
rem  @param %1 TaskID
rem  @param %2 UNC-Pfad
rem  @param %3 Zielverzeichnis
rem  ###################


rem #############################################################
rem
rem Einlesen der globalen Konfiguration
rem

SET glob_konfig_bat=..\konfig.bat

if exist %glob_konfig_bat% (
	call %glob_konfig_bat%
	cd %~dp0
) else (
	echo %glob_konfig_bat% nicht gefunden.
	exit	
)
rem #################################
rem 
rem  die folgenden Variablen werden 
rem  aus KONFIG.BAT bentigt
rem
rem  %glob_robo_errorlog%
rem  %glob_reportfile%
rem  %glob_completelog%
rem  %glob_reportdir%
rem  %glob_robo_statlog%
rem
rem #################################


rem  #############################################################
rem
rem  Festlegen der Logdateien fr den aktuellen ROBOCOPY-Task
rem
rem  TASKLOG:     alle wichtigen Informationen des aktuellen 
rem               Robocopy-Tasks
rem
rem  ROBOLOGFILE: Logfile fr den aktuellen Robocopy-Task
rem		  wird nach beenden in das entsprechende
rem		  Backup-Verzeichnis kopiert 
rem


SET TASKLOG=zz_tmp_robocopy_task%1.txt

SET ROBOLOGFILE=zz_tmp_ROBOCOPY_LOG%1


rem  
rem  Ausfuehren verhindern wenn Lock-Datei vorhanden
rem  

IF EXIST zz_tmp_robocopy_task_lock_%1 EXIT


rem  
rem  Lock-Datei anlegen
rem  


ECHO %~2 >> zz_tmp_usedshares
ECHO %~2 > zz_tmp_robocopy_task_lock_%1
echo %DATE% >> zz_tmp_robocopy_task_lock_%1
echo %TIME% >> zz_tmp_robocopy_task_lock_%1


rem -----------------
rem Startzeit setzen:
rem -----------------

set /a start_time=%time:~0,2%*3600+((1%time:~3,2%-100)*60)+(1%time:~6,2%-100)

rem  
rem  Quellverzeichnisse setzen (ohne Anfhrungszeichen)
rem  

SET SRC=%~2

rem  
rem  Zielverzeichnis ermitteln
rem  


SET DST=%3%SRC:~2%


echo --------------------
echo  starte ROBOCOPY %1
echo --------------------
echo.
echo Source=%SRC%
echo Destin=%DST%
echo.
echo --------------------


rem ######################
rem #                    #
rem #  Robocopy starten  #
rem #                    #
rem ######################


net USE %~2 > nul 2>> %TASKLOG%

rem %robocopy_exe% /MIR /COPYALL /R:0 /W:0 /NP /NDL /LOG:%ROBOLOGFILE% "%SRC%" "%DST%" /XD "System Volume Information"> nul

%robocopy_exe% /MIR /COPY:DATSO /MT:32 /R:0 /W:0 /NP /NDL /LOG:%ROBOLOGFILE% "%SRC%" "%DST%" /XD "System Volume Information"> nul

if errorlevel 1 (
	echo ROBOCOPY %1 beendet mit ERRORLEVEL %ERRORLEVEL%
) else (
	echo ROBOCOPY %1 beendet
)

rem ----------------
rem Ende-Zeit setzen
rem ----------------

set /a end_time=%time:~0,2%*3600+((1%time:~3,2%-100)*60)+(1%time:~6,2%-100)


rem  
rem  Schreibt Logdatei in Ueberordner von %DST%. "Verschandelt" damit 
rem  nicht die Original struktur und man kann es dennoch einfach zuordnen.
rem  Fuehren mehrere Sicherungen zur selben Logdatei, werden die Logdaten 
rem  jeweils angehaengt.
rem  
rem  Alte Logs werden bereits von _shares_spiegeln.bat vernichtet.
rem  
rem  

ECHO. 				>> %ROBOLOGFILE%
ECHO.				>> %ROBOLOGFILE%
ECHO ########EOF######### 	>> %ROBOLOGFILE%
ECHO. 				>> %ROBOLOGFILE%


call write_to_logfile.cmd %ROBOLOGFILE% "%DST%"\..\ROBOCOPY_LOG.txt




rem  ########################################################################
rem
rem  Auslesen und Loggen der Wichtigsten Daten des aktuellen Tasks
rem
rem


set LOGTIME=%DATE% %TIME:~0,8%

echo --- > %TASKLOG%
echo "%~2" >> %TASKLOG%

rem  ##################
rem    Anzahl Dateien

findstr "Dateien:" %ROBOLOGFILE% >nul
if %ERRORLEVEL% equ 0 (
	for /f "tokens=2,3,6" %%i in ('findstr "Dateien:" %ROBOLOGFILE%') do (
		echo %%j von %%i Dateien kopiert, %%k Fehler, >> %TASKLOG%
	)
) else (
	echo FEHLER: kein Zugriff auf Share >> %TASKLOG%
	goto :write_log
)



rem  ##################
rem     Anzahl Bytes

for /f "tokens=2-5" %%i in ('findstr "Bytes:" %ROBOLOGFILE%') do (
	
	set wert_c=%%k
	set einh_c=%%l
	set wert_t=%%i
	set einh_t=%%j

	if !wert_C! == 0 set einh_c=!einh_t!

	if not "!einh_c!" == "k" (
		if not "!einh_c!" == "m" (
			if not "!einh_c!" == "g" (
				if not "!einh_c!" == "t" (
					set einh_c=b
				)
			)
		)
	)
		 
	echo !wert_c! !einh_c! von !wert_t! !einh_t! kopiert
	
	echo !wert_c! !einh_c! von !wert_t! !einh_t! kopiert, >> %TASKLOG%
)


rem  ##################
rem    Dauer Robocopy (Robocopy-Zeitmessung "eigenwillig")
rem for /f "tokens=2" %%i in ('findstr "Zeiten:" %ROBOLOGFILE%') do ( echo Dauer %%i >> %TASKLOG% )

rem  ##################
rem    Dauer Systemzeit

set /a rbc_timediff=%end_time%-%start_time%
if %rbc_timediff% lss 0 (set /a rbc_timediff=%rbc_timediff%+86400)

set /a zzSTD=%rbc_timediff% / 3600
set /a zzMIN=%rbc_timediff% %% 3600 / 60
set /a zzSEK=%rbc_timediff% %% 3600 %% 60

if %zzSTD% lss 10 (set zzSTD=0%zzSTD%)
if %zzMIN% lss 10 (set zzMIN=0%zzMIN%)
if %zzSEK% lss 10 (set zzSEK=0%zzSEK%)

set rbc_timediff=%zzSTD%:%zzMIN%:%zzSEK%

echo Dauer %rbc_timediff% >> %TASKLOG%


:write_log
rem  ##################
echo --- >> %TASKLOG%

rem -----------------------------------------------
rem    Rauschreiben in die Report-Datei f. EMail
rem   (Informationen pro Share in kompakter Form)
rem

call write_to_logfile.cmd %TASKLOG% %glob_reportfile%


rem -----------------------------------------------
rem    Umformatieren der Logdaten auf eine Zeile
rem

rem	set _logtext=%LOGTIME% 
rem	for /f "delims= skip=1" %%a in (%TASKLOG%) do (
rem		if not "%%a"=="--- " set _logtext=!_logtext!%%a
rem	)
rem	echo !_logtext! > %TASKLOG%

set _logtext=
for /f "delims= skip=1" %%a in (%TASKLOG%) do (
	if not "%%a"=="--- " set _logtext=!_logtext!%%a
)

rem #--------------------------------- 
rem   Rausschreiben fr den Emailtext

echo !_logtext! > %TASKLOG%

call write_to_logfile.cmd %TASKLOG% %glob_robo_statlog%


rem #---------------------------------
rem   Rausschreiben in die Logdatei 
rem       (mit Datum/Uhrzeit)

echo %LOGTIME% !_logtext! > %TASKLOG%

call write_to_logfile.cmd %TASKLOG% %glob_completelog%




rem  ########################################################################
rem
rem  Auslesen und Verarbeiten der Fehlermeldungen
rem
SET TEMPFILE=zz_tmp_robo%1_errorlog.txt

echo. >> %TEMPFILE%
echo ----------------------------------------------------------- >> %TEMPFILE%
echo  Fehler beim Spiegeln der Daten von %~2 >> %TEMPFILE%
echo ----------------------------------------------------------- >> %TEMPFILE%
echo. >> %TEMPFILE%
findstr "FEHLER.[0-9]*.(0x" %ROBOLOGFILE% >> %TEMPFILE%
if errorlevel 1 (
	echo - KEINE - >> %TEMPFILE%
) else	(
	echo. 		>> %TEMPFILE%
	echo --- 	>> %TEMPFILE%
	call write_to_logfile.cmd %TEMPFILE% %glob_robo_errorlog%
)

rem  ########################################################################

rem
rem  Lschen der Logdatei im Arbeitverzeichnis
rem

DEL %ROBOLOGFILE%


rem  
rem  Aufrumen und Beenden
rem  

DEL %TASKLOG%
DEL %TEMPFILE%
DEL zz_tmp_robocopy_task_lock_%1

endlocal
rem  #######################################################################
rem  der folgende EXIT-Befehl ist wichtig!
rem

EXIT


